@@ -36,7 +36,7 @@ int progressbar_init(int N);
 // adds one '+' to progress bar
 inline void progressbar_print() {
  REprintf("+");
-// R_FlushConsole();
+ R_FlushConsole();
 }
 
 // finalizes progress bar
